import( kern crt )

( ********************************** )
( ** NAXOS 2.1x GRAPHICS LIBRARY  ** )
( ** using Video BIOS INT $10     ** )
( ** (C) 1992 by A.Zissis         ** )
( ** NOT for HERCULES Adaptors    ** )
( ********************************** )

( Standard Modes available on CGA/EGA/VGA )
const 320x200x4    *$0004 ;
const 320x200x16   *$010D ;
const 320x200x256  *$0213 ;
const 640x200x2    *$0306 ;
const 640x200x16   *$040E ;
const 640x350x4    *$050F ;
const 640x350x16   *$0610 ;
const 640x480x2    *$0711 ;
const 640x480x16   *$0812 ;

( SVGA modes on ET4000 based systems )
const 640x350x256  *$092D ;
const 640x400x256  *$0A2F ;
const 640x480x256  *$0B2E ;
const 800x600x16   *$0C29 ;
const 800x600x256  *$0D30 ;
const 1024x768x16  *$0E37 ;
const 1024x768x256 *$0F38 ;

const 80x24x16     *$1003 ;

(   devcaps x-res y-res pages colors )
var devcaps [ *320 *200 01 003
	      *320 *200 08 015 
	      *320 *200 01 255 
	      *640 *200 01 001
	      *640 *200 04 015
	      *640 *350 02 004
	      *640 *350 02 015
	      *640 *480 01 001
	      *640 *480 01 015
	      *640 *350 01 255
	      *640 *400 01 255
	      *640 *480 01 255	      
	      *800 *600 01 015
	      *800 *600 01 255
	     *1024 *768 01 015 	      
	     *1024 *768 01 255 
	       *80 *024 08 015 ] ;

var pix_x_max [ *80 ] ;
var pix_y_max [ *24 ] ;
var page_max  [ 8 ] ;
var color_max [ *16 ] ;

var page      [ 0 ] ;   
var pix_x     [ *0 ] ;  
var pix_y     [ *0 ] ;  


macro g$. _$. ;

: setmode ( switch to graphics mode specified in bl -> mode constants )
  b>a $10 intr
;

: setmodenc ( as above with no clear screen )
  b>a $80 or $10 intr
;


: graphmode ( switch to graphicsmode / input: mode.const in BX ) 
  bpush B>a split b>a 6 * devcaps adrn+ to pix_x_max , 6 i/copy
  apop split $10 intr
;

: graphmodenc ( switch to graphicsmode / input: mode.const in BX ) 
  bpush B>a split b>a devcaps adrn+ to pix_x_max , 6 i/copy
  apop split $80 or $10 intr
;

: textmode ( switch to standard 80x25 Textmode )
  80x24x16 graphmode  
;

: textmodenc ( as above with no clear screen )
  80x24x16 graphmodenc  
;


: entrymode ( restore initial videomode as on program entry )
  video_byte m>bl setmode
;

: setcolor ( set pixel color as in bx for 256Color Modes in others use fcolor )
  v-atrib bl>m
;

: disp_page ( select video page to display as specified in bl )
  b>a $5 combine $10 intr
;

: plot_page ( select video page to plot into as spec. in bl )
  page bl>m
;



: putpixel ( x-coord in bx; y-coord in ax; color is taken from pix_color )
  d.cx b>m pix_x b>m 
  d.dx a>m pix_y a>m
  page m>bl , 0 combine d.bx a>m 
  v-atrib m>al $0c combine $10 intr
;
  
: getpixel ( x-coord in bx; y-coord in ax; color is returned in al )
  d.cx b>m pix_x b>m 
  d.dx a>m pix_y a>m
  page m>bl , 0 combine d.bx a>m 
  A= $0D00 $10 intr
  d.ax m>al
;

: hline ( x1-coord in bx;y coord in ax; x2-coord in di )
  putpixel
  d>a pix_x m>b - a>b FOR
    pix_x m>b b+ b>m pix_y m>a putpixel
  next
;


: vline ( x1-coord in bx;y coord in ax; y2-coord in di )
  putpixel
  d>a pix_y m>b - a>b FOR
    pix_x m>b pix_y m>a 1+ a>m putpixel
  next
;

var BG_x1 word ;
var BG_y1 word ;
var BG_xd word ;
var BG_yd word ;
var BG_xf word ;


: line  ( input: x1: BX, y1: AX, x2: ES, y2: DI )
	( output: - )
	( CX is modified ! )
	
[ $89 $DA ] d>b
[ $89 $1E BG_y1 $E8 $96 $00 $89 $0E BG_yd $E8 $AE ]
[ $00 $92 ]
e>b
[ $89 $1E BG_x1 $E8 $85 $00 $89 $0E BG_xd $E8 $9D ]
[ $00 $8B $D8 $3B $DA $7E $3C $8B $C2 $03 $C0 $2B $C3 $A3 BG_xf ]
[ $8B $CB $41 $E8 $75 $00 $A1 BG_xf $0B $C0 $7E $15 $03 $C2 $03 ]
[ $C2 $2B $C3 $2B $C3 $A3 BG_xf $A1 BG_yd $01 $06 BG_y1 $EB ]
[ $08 $90 $03 $C2 $03 $C2 $A3 BG_xf $A1 BG_xd $01 $06 BG_x1 ]
[ $E2 $D1 $C3 $8B $C3 $03 $C0 $2B $C2 $A3 BG_xf $8B $CA $41 $E8 ]
[ $39 $00 $A1 BG_xf $0B $C0 $7E $15 $03 $C3 $03 $C3 $2B $C2 $2B ]
[ $C2 $A3 BG_xf $A1 BG_xd $01 $06 BG_x1 $EB $08 $90 $03 $C3 ]
[ $03 $C3 $A3 BG_xf $A1 BG_yd $01 $06 BG_y1 $E2 $D1 $C3 $33 ]
[ $C9 $2B $C3 $74 $05 $78 $02 $41 $C3 $49 $C3 $52 $53 $50 $8B ]
[ $1E BG_x1 $A1 BG_y1 ]
 putpixel
[ $58 $5B $5A $C3 ]
[ $0B $C0 $79 $02 $F7 $D8 $C3 ]
;


: lineto ( x in BX , y in AX )
  pix_x b>m pix_y a>m
  [ $06 $51 ] ( push es , push cx )
  a>d b>e
  pix_x m>b pix_y m>a 
  line
  [ $59 $07 ] ( pop cx , pop es )
;



: main init
  640x480x16 graphmode
  red setcolor
  0 pix_x_max m>a do
    i>b , 0 putpixel
    pix_x_max m>a i>b - a>b pix_y_max m>a lineto
  , 2 +loop
  wait
  textmode
  pix_x_max m>a u.d cr
  pix_y_max m>a u.d cr 
  page_max m>al u.d cr
  color_max m>al u.d cr  	  
  wait
;