{************************************************}
{                                                }
{   Turbo Pascal 6.0                             }
{   Turbo Vision Demo                            }
{   Copyright (c) 1990 by Borland International  }
{                                                }
{************************************************}

unit Rechner;

{$F+,O+,S-,D-}

{ Calculator object. See TVDEMO.PAS for an example
  program that uses this unit.
}

interface

uses Drivers, Objects, Views, Dialogs;

type
  str15 = string[15];

  TCalcState = (csFirst, csValid, csError);

  PCalcDisplay = ^TCalcDisplay;
  TCalcDisplay = object(TView)
    mode:byte;
    Status: TCalcState;
    Number: str15;
    Sign: Char;
    Operator: Char;
    Operand: longint;
    constructor Init(var Bounds: TRect);
    constructor Load(var S: TStream);
    procedure CalcKey(Key: Char);
    procedure Clear;
    procedure Draw; virtual;
    function GetPalette: PPalette; virtual;
    procedure HandleEvent(var Event: TEvent); virtual;
    procedure Store(var S: TStream);
  end;

  PCalculator = ^TCalculator;
  TCalculator = object(TDialog)
    constructor Init;
  end;

const
  RCalcDisplay: TStreamRec = (
     ObjType: 10040;
     VmtLink: Ofs(TypeOf(TCalcDisplay)^);
     Load:    @TCalcDisplay.Load;
     Store:   @TCalcDisplay.Store
  );
  RCalculator: TStreamRec = (
     ObjType: 10041;
     VmtLink: Ofs(TypeOf(TCalculator)^);
     Load:    @TCalculator.Load;
     Store:   @TCalculator.Store
  );

procedure RegisterCalc;

implementation

const
  cmCalcButton = 100;

constructor TCalcDisplay.Init(var Bounds: TRect);
begin
  TView.Init(Bounds);
  Options := Options or ofSelectable;
  EventMask := evKeyDown + evBroadcast;
  mode := 0;
  Clear;
end;

constructor TCalcDisplay.Load(var S: TStream);
begin
  TView.Load(S);
  S.Read(Status, SizeOf(Status) + SizeOf(Number) + SizeOf(Sign) +
    SizeOf(Operator) + SizeOf(Operand));
end;

procedure TCalcDisplay.CalcKey(Key: Char);
var
  R: longint;

procedure Error;
begin
  Status := csError;
  Number := 'Error';
  Sign := ' ';
end;

procedure hex(num:longint;var s:str15;var vz:char);
var x : integer;
    nw: longint;
    negf : boolean;
begin
 if num = 0 then begin
    s :='0'; vz := ' ';
 end
 else begin
  if mode = 2 then begin
    if num < 0 then vz := '-' else vz := ' ';
    s :='';
    num := abs(num);
    while num >0 do begin
      x := num mod 16;
      if x>9 then s := chr(55+x)+s
             else s := chr(48+x)+s;
      num := num div 16;
    end;
  end
  else begin
    nw := num;
    vz := ' ';
    s :='';
    if nw<0 then BEGIN nw := nw and $7FFFFFFF;
                       negf := true;
                 END else negf := false;

    while nw >0 do begin
      x := nw mod 16;
      if x>9 then s := chr(55+x)+s
             else s := chr(48+x)+s;
      nw := nw div 16;
      if negf then begin
         negf := false;
         nw := nw or $08000000;
      end;
    end;
  end;
 end;
end;

procedure SetDisplay(R: longint);
var
  S: string[63];
  rw : real;
begin
 rw := r;
 if r<0 then rw := rw+4294967295.0;
 if (mode and 2) = 0 then begin
  if mode=0 then Str(R, S)
            else Str(RW:0:0,s);
  if S[1] <> '-' then Sign := ' ' else
  begin
    Delete(S, 1, 1);
    Sign := '-';
  end;
  if Length(S) > 15 + 1 + 10 then Error
                             else Number := S;
 end else hex(r,number,sign);
end;

procedure GetDisplay(var R: longint);
var
  E: Integer;
  N: string[15];
  rv : real;
begin
  n := Number;
  if mode and 2 <>0 then n := '$'+n;
  if mode = 1 then begin
    Val(Sign + N, RV, E);
    if rv>2147483647.0 then rv := rv-4294967295.0;
    r := trunc(rv);
  end else val(sign+n,r,e);
end;

procedure CheckFirst;
begin
  if Status = csFirst then
  begin
    Status := csValid;
    Number := '0';
    Sign := ' ';
  end;
end;

begin
  Key := UpCase(Key);
  if (Status = csError) and (Key <> '#') then Key := ' ';
  case Key of
    '0'..'9':
      begin
        CheckFirst;
        if Number = '0' then Number := '';
        Number := Number + Key;
      end;
    'A'..'F':
      begin
        if mode and 2 <> 0 then begin
          CheckFirst;
          if Number = '0' then Number := '';
          Number := Number + Key;
        end;
      end;
    '_', #241:
      if Sign = ' ' then Sign := '-' else Sign := ' ';
    '$': begin
           getdisplay(r);
           inc(mode);
           mode := mode and 3;
           setdisplay(r);
         end;
    '+', '-', '*', '/', '=', '\','&','|','<','>','X', #13:
      begin
        if Status = csValid then
        begin
          Status := csFirst;
          GetDisplay(R);
          case Operator of
            '&': SetDisplay(Operand and R);
            '|': SetDisplay(Operand or R);
            'X': SetDisplay(Operand xor R);
            '<': SetDisplay(Operand shl R);
            '>': SetDisplay(Operand shr R);
            '+': SetDisplay(Operand + R);
            '-': SetDisplay(Operand - R);
            '*': SetDisplay(Operand * R);
            '\': if R = 0 then Error else SetDisplay(Operand mod R);
            '/': if R = 0 then Error else SetDisplay(Operand div R);
          end;
        end;
        Operator := Key;
        GetDisplay(Operand);
      end;
    '#':
      Clear;
  end;
  DrawView;
end;

procedure TCalcDisplay.Clear;
begin
  Status := csFirst;
  Number := '0';
  Sign := ' ';
  Operator := '=';
end;

procedure TCalcDisplay.Draw;
var
  Color: Byte;
  I: Integer;
  B: TDrawBuffer;
  s: string[2];
begin
  Color := GetColor(1);
  I := Size.X - Length(Number) - 2;
  MoveChar(B, ' ', Color, Size.X);
  case mode of
  0: s := ' S';
  1: s := ' U';
  2: s := '$S';
  3: s := '$U';
  end;
  MoveStr(B[1], S, Color);
  MoveChar(B[I], Sign, Color, 1);
  MoveStr(B[I + 1], Number, Color);
  WriteBuf(0, 0, Size.X, 1, B);
end;

function TCalcDisplay.GetPalette: PPalette;
const
  P: string[1] = #19;
begin
  GetPalette := @P;
end;

procedure TCalcDisplay.HandleEvent(var Event: TEvent);
begin
  TView.HandleEvent(Event);
  case Event.What of
    evKeyDown:
      begin
        CalcKey(Event.CharCode);
        ClearEvent(Event);
      end;
    evBroadcast:
      if Event.Command = cmCalcButton then
      begin
        CalcKey(PButton(Event.InfoPtr)^.Title^[1]);
        ClearEvent(Event);
      end;
  end;
end;

procedure TCalcDisplay.Store(var S: TStream);
begin
  TView.Store(S);
  S.Write(Status, SizeOf(Status) + SizeOf(Number) + SizeOf(Sign) +
    SizeOf(Operator) + SizeOf(Operand));
end;

{ TCalculator }

constructor TCalculator.Init;
const
  KeyChar: array[0..29] of Char = 'DEF&#ABC|<789*>456/\123-'#241'0$=+X';
var
  I: Integer;
  P: PView;
  R: TRect;
begin
  R.Assign(5, 3, 39, 22);
  TDialog.Init(R, 'Hex-Rechner');
  Options := Options or ofFirstClick;
  for I := 0 to 29 do
  begin
    R.A.X := (I mod 5) * 6 + 2;
    R.A.Y := (I div 5) * 2 + 5;
    R.B.X := R.A.X + 5;
    R.B.Y := R.A.Y + 2;
    P := New(PButton, Init(R, KeyChar[I], cmCalcButton,
      bfNormal + bfBroadcast));
    P^.Options := P^.Options and not ofSelectable;
    Insert(P);
  end;
  R.Assign(3, 2, 31, 3);
  Insert(New(PCalcDisplay, Init(R)));
end;

procedure RegisterCalc;
begin
  RegisterType(RCalcDisplay);
  RegisterType(RCalculator);
end;

end.
